@@ -443,7 +443,7 @@ def mlp_forward(self, hidden_states):
     with on_device(router_logits.device):
         routing_data, gather_idx, scatter_idx = routing(router_logits, self.router.top_k)
 
-    routed_out = self.experts(hidden_states, routing_data, gather_idx, scatter_idx)
+    routed_out = self.experts(hidden_states, routing_data, gather_idx, scatter_idx=scatter_idx)
     routed_out = routed_out.reshape(batch_size, -1, self.router.hidden_dim)
     return routed_out, router_logits
 
