@@ -1156,6 +1156,8 @@ def gather_full_tensor(local_tensor: torch.Tensor, shard_dim: int, device_mesh)
         The full reconstructed tensor (same on all ranks)
     """
     world_size = device_mesh.size()
+    # In case of TP+DP configuration, the TP group should be used for gathering, not the full DP group
+    process_group = device_mesh.get_group("tp") if "tp" in device_mesh.mesh_dim_names else None
 
     # Normalize negative dimension
     if shard_dim < 0:
@@ -1163,7 +1165,7 @@ def gather_full_tensor(local_tensor: torch.Tensor, shard_dim: int, device_mesh)
 
     # Gather all shards
     gathered_tensors = [torch.empty_like(local_tensor) for _ in range(world_size)]
-    dist.all_gather(gathered_tensors, local_tensor.contiguous())
+    dist.all_gather(gathered_tensors, local_tensor.contiguous(), group=process_group)
 
     # Concatenate along the shard dimension
     return torch.cat(gathered_tensors, dim=shard_dim)
