@@ -263,13 +263,16 @@ class LightOnOcrProcessor(ProcessorMixin):
             images_kwargs.update(kwargs)
 
             size = images_kwargs.get("size", None) or self.image_processor.size
+            patch_size = images_kwargs.get("patch_size", None) or self.image_processor.patch_size
+            if isinstance(patch_size, dict) and "height" in patch_size and "width" in patch_size:
+                patch_size = (patch_size["height"], patch_size["width"])
 
             num_image_tokens = []
             for height, width in image_sizes:
                 resized_height, resized_width = get_resize_output_image_size(
                     np.zeros((height, width, 3)),
                     size=(size["longest_edge"], size["longest_edge"]),
-                    patch_size=(self.effective_patch_size, self.effective_patch_size),
+                    patch_size=patch_size,
                 )
                 num_height_tokens = resized_height // self.effective_patch_size
                 num_width_tokens = resized_width // self.effective_patch_size
