@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright 2018 Hao Tan, Mohit Bansal, and the HuggingFace team
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,15 +14,13 @@
 """PyTorch LXMERT model."""
 
 import math
-import os
-import warnings
 from dataclasses import dataclass
-from typing import Optional, Union
 
 import torch
 from torch import nn
 from torch.nn import CrossEntropyLoss, SmoothL1Loss
 
+from ... import initialization as init
 from ...activations import ACT2FN, gelu
 from ...modeling_utils import PreTrainedModel
 from ...utils import ModelOutput, auto_docstring, logging
@@ -78,14 +75,14 @@ class LxmertModelOutput(ModelOutput):
         the self-attention heads.
     """
 
-    language_output: Optional[torch.FloatTensor] = None
-    vision_output: Optional[torch.FloatTensor] = None
-    pooled_output: Optional[torch.FloatTensor] = None
-    language_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    vision_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    language_attentions: Optional[tuple[torch.FloatTensor]] = None
-    vision_attentions: Optional[tuple[torch.FloatTensor]] = None
-    cross_encoder_attentions: Optional[tuple[torch.FloatTensor]] = None
+    language_output: torch.FloatTensor | None = None
+    vision_output: torch.FloatTensor | None = None
+    pooled_output: torch.FloatTensor | None = None
+    language_hidden_states: tuple[torch.FloatTensor] | None = None
+    vision_hidden_states: tuple[torch.FloatTensor] | None = None
+    language_attentions: tuple[torch.FloatTensor] | None = None
+    vision_attentions: tuple[torch.FloatTensor] | None = None
+    cross_encoder_attentions: tuple[torch.FloatTensor] | None = None
 
 
 @dataclass
@@ -121,13 +118,13 @@ class LxmertForQuestionAnsweringOutput(ModelOutput):
         the self-attention heads.
     """
 
-    loss: Optional[torch.FloatTensor] = None
-    question_answering_score: Optional[torch.FloatTensor] = None
-    language_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    vision_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    language_attentions: Optional[tuple[torch.FloatTensor]] = None
-    vision_attentions: Optional[tuple[torch.FloatTensor]] = None
-    cross_encoder_attentions: Optional[tuple[torch.FloatTensor]] = None
+    loss: torch.FloatTensor | None = None
+    question_answering_score: torch.FloatTensor | None = None
+    language_hidden_states: tuple[torch.FloatTensor] | None = None
+    vision_hidden_states: tuple[torch.FloatTensor] | None = None
+    language_attentions: tuple[torch.FloatTensor] | None = None
+    vision_attentions: tuple[torch.FloatTensor] | None = None
+    cross_encoder_attentions: tuple[torch.FloatTensor] | None = None
 
 
 @dataclass
@@ -168,94 +165,15 @@ class LxmertForPreTrainingOutput(ModelOutput):
         the self-attention heads.
     """
 
-    loss: Optional[torch.FloatTensor] = None
-    prediction_logits: Optional[torch.FloatTensor] = None
-    cross_relationship_score: Optional[torch.FloatTensor] = None
-    question_answering_score: Optional[torch.FloatTensor] = None
-    language_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    vision_hidden_states: Optional[tuple[torch.FloatTensor]] = None
-    language_attentions: Optional[tuple[torch.FloatTensor]] = None
-    vision_attentions: Optional[tuple[torch.FloatTensor]] = None
-    cross_encoder_attentions: Optional[tuple[torch.FloatTensor]] = None
-
-
-def load_tf_weights_in_lxmert(model, config, tf_checkpoint_path):
-    """Load tf checkpoints in a pytorch model."""
-    try:
-        import re
-
-        import numpy as np
-        import tensorflow as tf
-    except ImportError:
-        logger.error(
-            "Loading a TensorFlow model in PyTorch, requires TensorFlow to be installed. Please see "
-            "https://www.tensorflow.org/install/ for installation instructions."
-        )
-        raise
-    tf_path = os.path.abspath(tf_checkpoint_path)
-    logger.info(f"Converting TensorFlow checkpoint from {tf_path}")
-    # Load weights from TF model
-    init_vars = tf.train.list_variables(tf_path)
-    names = []
-    arrays = []
-    for name, shape in init_vars:
-        logger.info(f"Loading TF weight {name} with shape {shape}")
-        array = tf.train.load_variable(tf_path, name)
-        names.append(name)
-        arrays.append(array)
-
-    for name, array in zip(names, arrays):
-        name = name.split("/")
-        # adam_v and adam_m are variables used in AdamWeightDecayOptimizer to calculated m and v
-        # which are not required for using pretrained model
-        if any(
-            n
-            in [
-                "adam_v",
-                "adam_m",
-                "AdamWeightDecayOptimizer",
-                "AdamWeightDecayOptimizer_1",
-                "global_step",
-            ]
-            for n in name
-        ):
-            logger.info(f"Skipping {'/'.join(name)}")
-            continue
-        pointer = model
-        for m_name in name:
-            if re.fullmatch(r"[A-Za-z]+_\d+", m_name):
-                scope_names = re.split(r"_(\d+)", m_name)
-            else:
-                scope_names = [m_name]
-            if scope_names[0] == "kernel" or scope_names[0] == "gamma":
-                pointer = getattr(pointer, "weight")
-            elif scope_names[0] == "output_bias" or scope_names[0] == "beta":
-                pointer = getattr(pointer, "bias")
-            elif scope_names[0] == "output_weights":
-                pointer = getattr(pointer, "weight")
-            elif scope_names[0] == "squad":
-                pointer = getattr(pointer, "classifier")
-            else:
-                try:
-                    pointer = getattr(pointer, scope_names[0])
-                except AttributeError:
-                    logger.info(f"Skipping {'/'.join(name)}")
-                    continue
-            if len(scope_names) >= 2:
-                num = int(scope_names[1])
-                pointer = pointer[num]
-        if m_name[-11:] == "_embeddings":
-            pointer = getattr(pointer, "weight")
-        elif m_name == "kernel":
-            array = np.transpose(array)
-        try:
-            assert pointer.shape == array.shape
-        except AssertionError as e:
-            e.args += (pointer.shape, array.shape)
-            raise
-        logger.info(f"Initialize PyTorch weight {name}")
-        pointer.data = torch.from_numpy(array)
-    return model
+    loss: torch.FloatTensor | None = None
+    prediction_logits: torch.FloatTensor | None = None
+    cross_relationship_score: torch.FloatTensor | None = None
+    question_answering_score: torch.FloatTensor | None = None
+    language_hidden_states: tuple[torch.FloatTensor] | None = None
+    vision_hidden_states: tuple[torch.FloatTensor] | None = None
+    language_attentions: tuple[torch.FloatTensor] | None = None
+    vision_attentions: tuple[torch.FloatTensor] | None = None
+    cross_encoder_attentions: tuple[torch.FloatTensor] | None = None
 
 
 class LxmertEmbeddings(nn.Module):
@@ -267,8 +185,6 @@ class LxmertEmbeddings(nn.Module):
         self.position_embeddings = nn.Embedding(config.max_position_embeddings, config.hidden_size, padding_idx=0)
         self.token_type_embeddings = nn.Embedding(config.type_vocab_size, config.hidden_size, padding_idx=0)
 
-        # self.LayerNorm is not snake-cased to stick with TensorFlow model variable name and be able to load
-        # any TensorFlow checkpoint file
         self.LayerNorm = nn.LayerNorm(config.hidden_size, eps=1e-12)
         self.dropout = nn.Dropout(config.hidden_dropout_prob)
 
@@ -679,19 +595,11 @@ class LxmertPredictionHeadTransform(nn.Module):
 
 
 class LxmertLMPredictionHead(nn.Module):
-    def __init__(self, config, lxmert_model_embedding_weights):
+    def __init__(self, config):
         super().__init__()
         self.transform = LxmertPredictionHeadTransform(config)
-
-        # The output weights are the same as the input embeddings, but there is
-        # an output-only bias for each token.
-        self.decoder = nn.Linear(
-            lxmert_model_embedding_weights.size(1),
-            lxmert_model_embedding_weights.size(0),
-            bias=False,
-        )
-        self.decoder.weight = lxmert_model_embedding_weights
-        self.bias = nn.Parameter(torch.zeros(lxmert_model_embedding_weights.size(0)))
+        self.decoder = nn.Linear(config.hidden_size, config.vocab_size, bias=False)
+        self.bias = nn.Parameter(torch.zeros(config.vocab_size))
 
     def forward(self, hidden_states):
         hidden_states = self.transform(hidden_states)
@@ -746,9 +654,9 @@ class LxmertVisualObjHead(nn.Module):
 
 
 class LxmertPreTrainingHeads(nn.Module):
-    def __init__(self, config, lxmert_model_embedding_weights):
+    def __init__(self, config):
         super().__init__()
-        self.predictions = LxmertLMPredictionHead(config, lxmert_model_embedding_weights)
+        self.predictions = LxmertLMPredictionHead(config)
         self.seq_relationship = nn.Linear(config.hidden_size, 2)
 
     def forward(self, sequence_output, pooled_output):
@@ -760,27 +668,15 @@ class LxmertPreTrainingHeads(nn.Module):
 @auto_docstring
 class LxmertPreTrainedModel(PreTrainedModel):
     config: LxmertConfig
-    load_tf_weights = load_tf_weights_in_lxmert
     base_model_prefix = "lxmert"
-    _supports_param_buffer_assignment = False
+    input_modalities = ("image", "text")
 
+    @torch.no_grad()
     def _init_weights(self, module):
         """Initialize the weights"""
-        if isinstance(module, nn.Linear):
-            # Slightly different from the TF version which uses truncated_normal for initialization
-            # cf https://github.com/pytorch/pytorch/pull/5617
-            module.weight.data.normal_(mean=0.0, std=self.config.initializer_range)
-            if module.bias is not None:
-                module.bias.data.zero_()
-        elif isinstance(module, nn.Embedding):
-            module.weight.data.normal_(mean=0.0, std=self.config.initializer_range)
-            if module.padding_idx is not None:
-                module.weight.data[module.padding_idx].zero_()
-        elif isinstance(module, nn.LayerNorm):
-            module.bias.data.zero_()
-            module.weight.data.fill_(1.0)
-        elif isinstance(module, LxmertLMPredictionHead):
-            module.bias.data.zero_()
+        super()._init_weights(module)
+        if isinstance(module, LxmertLMPredictionHead):
+            init.zeros_(module.bias)
 
 
 @auto_docstring
@@ -802,17 +698,18 @@ class LxmertModel(LxmertPreTrainedModel):
     @auto_docstring
     def forward(
         self,
-        input_ids: Optional[torch.LongTensor] = None,
-        visual_feats: Optional[torch.FloatTensor] = None,
-        visual_pos: Optional[torch.FloatTensor] = None,
-        attention_mask: Optional[torch.FloatTensor] = None,
-        visual_attention_mask: Optional[torch.FloatTensor] = None,
-        token_type_ids: Optional[torch.LongTensor] = None,
-        inputs_embeds: Optional[torch.FloatTensor] = None,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
-    ) -> Union[LxmertModelOutput, tuple[torch.FloatTensor]]:
+        input_ids: torch.LongTensor | None = None,
+        visual_feats: torch.FloatTensor | None = None,
+        visual_pos: torch.FloatTensor | None = None,
+        attention_mask: torch.FloatTensor | None = None,
+        visual_attention_mask: torch.FloatTensor | None = None,
+        token_type_ids: torch.LongTensor | None = None,
+        inputs_embeds: torch.FloatTensor | None = None,
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
+        **kwargs,
+    ) -> LxmertModelOutput | tuple[torch.FloatTensor]:
         r"""
         visual_feats (`torch.FloatTensor` of shape `(batch_size, num_visual_features, visual_feat_dim)`):
             This input represents visual features. They ROI pooled object features from bounding boxes using a
@@ -935,7 +832,10 @@ class LxmertModel(LxmertPreTrainedModel):
 
 @auto_docstring
 class LxmertForPreTraining(LxmertPreTrainedModel):
-    _tied_weights_keys = ["cls.predictions.decoder.weight"]
+    # help saving them
+    _tied_weights_keys = {
+        "cls.predictions.decoder.weight": "lxmert.embeddings.word_embeddings.weight",
+    }
 
     def __init__(self, config):
         super().__init__(config)
@@ -954,7 +854,7 @@ class LxmertForPreTraining(LxmertPreTrainedModel):
         self.lxmert = LxmertModel(config)
 
         # Pre-training heads
-        self.cls = LxmertPreTrainingHeads(config, self.lxmert.embeddings.word_embeddings.weight)
+        self.cls = LxmertPreTrainingHeads(config)
         if self.task_obj_predict:
             self.obj_predict_head = LxmertVisualObjHead(config)
         if self.task_qa:
@@ -992,11 +892,8 @@ class LxmertForPreTraining(LxmertPreTrainedModel):
             }
         self.visual_losses = visual_losses
 
-    def _tie_weights(self):
-        self.cls.predictions.decoder.weight = self.lxmert.embeddings.word_embeddings.weight
-
     def resize_token_embeddings(
-        self, new_num_tokens: int, pad_to_multiple_of: Optional[int] = None, mean_resizing: bool = True
+        self, new_num_tokens: int, pad_to_multiple_of: int | None = None, mean_resizing: bool = True
     ) -> nn.Embedding:
         # Adding the following steps to resize bias to match the shape of resized embeddings
         new_embeddings = super().resize_token_embeddings(new_num_tokens, pad_to_multiple_of, mean_resizing)
@@ -1087,22 +984,22 @@ class LxmertForPreTraining(LxmertPreTrainedModel):
     @auto_docstring
     def forward(
         self,
-        input_ids: Optional[torch.LongTensor] = None,
-        visual_feats: Optional[torch.FloatTensor] = None,
-        visual_pos: Optional[torch.FloatTensor] = None,
-        attention_mask: Optional[torch.FloatTensor] = None,
-        visual_attention_mask: Optional[torch.FloatTensor] = None,
-        token_type_ids: Optional[torch.LongTensor] = None,
-        inputs_embeds: Optional[torch.FloatTensor] = None,
-        labels: Optional[torch.LongTensor] = None,
-        obj_labels: Optional[dict[str, tuple[torch.FloatTensor, torch.FloatTensor]]] = None,
-        matched_label: Optional[torch.LongTensor] = None,
-        ans: Optional[torch.Tensor] = None,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
+        input_ids: torch.LongTensor | None = None,
+        visual_feats: torch.FloatTensor | None = None,
+        visual_pos: torch.FloatTensor | None = None,
+        attention_mask: torch.FloatTensor | None = None,
+        visual_attention_mask: torch.FloatTensor | None = None,
+        token_type_ids: torch.LongTensor | None = None,
+        inputs_embeds: torch.FloatTensor | None = None,
+        labels: torch.LongTensor | None = None,
+        obj_labels: dict[str, tuple[torch.FloatTensor, torch.FloatTensor]] | None = None,
+        matched_label: torch.LongTensor | None = None,
+        ans: torch.Tensor | None = None,
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
         **kwargs,
-    ) -> Union[LxmertForPreTrainingOutput, tuple[torch.FloatTensor]]:
+    ) -> LxmertForPreTrainingOutput | tuple[torch.FloatTensor]:
         r"""
         visual_feats (`torch.FloatTensor` of shape `(batch_size, num_visual_features, visual_feat_dim)`):
             This input represents visual features. They ROI pooled object features from bounding boxes using a
@@ -1140,14 +1037,6 @@ class LxmertForPreTraining(LxmertPreTrainedModel):
             a one hot representation hof the correct answer *optional*
         """
 
-        if "masked_lm_labels" in kwargs:
-            warnings.warn(
-                "The `masked_lm_labels` argument is deprecated and will be removed in a future version, use `labels`"
-                " instead.",
-                FutureWarning,
-            )
-            labels = kwargs.pop("masked_lm_labels")
-
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
         device = input_ids.device if input_ids is not None else inputs_embeds.device
@@ -1334,18 +1223,19 @@ class LxmertForQuestionAnswering(LxmertPreTrainedModel):
     @auto_docstring
     def forward(
         self,
-        input_ids: Optional[torch.LongTensor] = None,
-        visual_feats: Optional[torch.FloatTensor] = None,
-        visual_pos: Optional[torch.FloatTensor] = None,
-        attention_mask: Optional[torch.FloatTensor] = None,
-        visual_attention_mask: Optional[torch.FloatTensor] = None,
-        token_type_ids: Optional[torch.LongTensor] = None,
-        inputs_embeds: Optional[torch.FloatTensor] = None,
-        labels: Optional[torch.Tensor] = None,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
-    ) -> Union[LxmertForQuestionAnsweringOutput, tuple[torch.FloatTensor]]:
+        input_ids: torch.LongTensor | None = None,
+        visual_feats: torch.FloatTensor | None = None,
+        visual_pos: torch.FloatTensor | None = None,
+        attention_mask: torch.FloatTensor | None = None,
+        visual_attention_mask: torch.FloatTensor | None = None,
+        token_type_ids: torch.LongTensor | None = None,
+        inputs_embeds: torch.FloatTensor | None = None,
+        labels: torch.Tensor | None = None,
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
+        **kwargs,
+    ) -> LxmertForQuestionAnsweringOutput | tuple[torch.FloatTensor]:
         r"""
         visual_feats (`torch.FloatTensor` of shape `(batch_size, num_visual_features, visual_feat_dim)`):
             This input represents visual features. They ROI pooled object features from bounding boxes using a
