@@ -83,8 +83,8 @@ def eager_attention_forward(
         scaling = query.size(-1) ** -0.5
 
     attn_weights = torch.matmul(query, key.transpose(2, 3)) * scaling
-    if attention_mask is not None and attention_mask.ndim == 4:
-        attn_weights = attn_weights + attention_mask[:, :, :, : key.shape[-2]]
+    if attention_mask is not None:
+        attn_weights = attn_weights + attention_mask
 
     attn_weights = nn.functional.softmax(attn_weights, dim=-1)
 
@@ -763,7 +763,7 @@ class Qwen2AudioForConditionalGeneration(Qwen2AudioPreTrainedModel, GenerationMi
 
                 audio_attention_mask = create_bidirectional_mask(
                     config=self.audio_tower.config,
-                    input_embeds=dummy_embeds,
+                    inputs_embeds=dummy_embeds,
                     attention_mask=audio_attention_mask_2d,
                 )
 
