@@ -94,6 +94,62 @@ class Sam3TrackerMaskDecoderConfig(Sam2MaskDecoderConfig):
 
 
 class Sam3TrackerConfig(Sam2Config):
+    r"""
+    [`Sam3TrackerConfig`] is the configuration class to store the configuration of a [`Sam3TrackerModel`]. It is used to instantiate a
+    SAM3_TRACKER model according to the specified arguments, defining the memory attention, memory encoder, and image encoder
+    configs. Instantiating a configuration defaults will yield a similar configuration to that of the SAM 2.1 Hiera-tiny
+    [facebook/sam3_tracker.1-hiera-tiny](https://huggingface.co/facebook/sam3_tracker.1-hiera-tiny) architecture.
+
+    Configuration objects inherit from [`PreTrainedConfig`] and can be used to control the model outputs. Read the
+    documentation from [`PreTrainedConfig`] for more information.
+
+    <Tip>
+
+    SAM3 Tracker checkpoints with `model_type="sam3_tracker_video"` are compatible with `Sam3TrackerModel` since the
+    video variant weights are a superset of the image-only model weights. You may see a warning about model type
+    mismatch when loading such checkpoints, which can be safely ignored in this case.
+
+    </Tip>
+
+    Args:
+        vision_config (Union[`dict`, `Sam3TrackerVisionConfig`], *optional*):
+            Dictionary of configuration options used to initialize [`Sam3TrackerVisionConfig`].
+        prompt_encoder_config (Union[`dict`, `Sam3TrackerPromptEncoderConfig`], *optional*):
+            Dictionary of configuration options used to initialize [`Sam3TrackerPromptEncoderConfig`].
+        mask_decoder_config (Union[`dict`, `Sam3TrackerMaskDecoderConfig`], *optional*):
+            Dictionary of configuration options used to initialize [`Sam3TrackerMaskDecoderConfig`].
+        initializer_range (`float`, *optional*, defaults to 0.02):
+            Standard deviation for parameter initialization.
+
+    Example:
+
+    ```python
+    >>> from transformers import (
+    ...     Sam3TrackerVisionConfig,
+    ...     Sam3TrackerPromptEncoderConfig,
+    ...     Sam3TrackerMaskDecoderConfig,
+    ...     Sam3TrackerModel,
+    ... )
+
+    >>> # Initializing a Sam3TrackerConfig with `"facebook/sam3_tracker.1_hiera_tiny"` style configuration
+    >>> configuration = Sam3TrackerConfig()
+
+    >>> # Initializing a Sam3TrackerModel (with random weights) from the `"facebook/sam3_tracker.1_hiera_tiny"` style configuration
+    >>> model = Sam3TrackerModel(configuration)
+
+    >>> # Accessing the model configuration
+    >>> configuration = model.config
+
+    >>> # We can also initialize a Sam3TrackerConfig from a Sam3TrackerVisionConfig, Sam3TrackerPromptEncoderConfig, and Sam3TrackerMaskDecoderConfig
+    >>> # Initializing SAM3_TRACKER vision encoder, memory attention, and memory encoder configurations
+    >>> vision_config = Sam3TrackerVisionConfig()
+    >>> prompt_encoder_config = Sam3TrackerPromptEncoderConfig()
+    >>> mask_decoder_config = Sam3TrackerMaskDecoderConfig()
+
+    >>> config = Sam3TrackerConfig(vision_config, prompt_encoder_config, mask_decoder_config)
+    ```
+    """
+
     def __init__(
         self,
         vision_config=None,
