@@ -44,7 +44,8 @@ from ...utils import (
     logging,
     torch_compilable_check,
 )
-from ...utils.generic import OutputRecorder, check_model_inputs
+from ...utils.generic import merge_with_config_defaults
+from ...utils.output_capturing import OutputRecorder, capture_outputs
 from ..auto import AutoModel
 from ..gemma3.configuration_gemma3 import Gemma3Config, Gemma3TextConfig
 from ..gemma3.modeling_gemma3 import (
@@ -828,7 +829,8 @@ class T5Gemma2TextEncoder(T5Gemma2PreTrainedModel):
         # Initialize weights and apply final processing
         self.post_init()
 
-    @check_model_inputs
+    @merge_with_config_defaults
+    @capture_outputs
     @auto_docstring
     def forward(
         self,
@@ -855,7 +857,7 @@ class T5Gemma2TextEncoder(T5Gemma2PreTrainedModel):
         if not isinstance(self_attn_mask_mapping := attention_mask, dict):
             mask_kwargs = {
                 "config": self.config,
-                "input_embeds": inputs_embeds,
+                "inputs_embeds": inputs_embeds,
                 "attention_mask": attention_mask,
             }
             self_attn_mask_mapping = {
@@ -960,7 +962,6 @@ class T5Gemma2Encoder(T5Gemma2PreTrainedModel):
         )
         return special_image_mask
 
-    @check_model_inputs
     @auto_docstring
     def forward(
         self,
@@ -989,16 +990,13 @@ class T5Gemma2Encoder(T5Gemma2PreTrainedModel):
 
             inputs_embeds = inputs_embeds.masked_scatter(image_mask, image_features)
 
-        hidden_states = self.text_model(
+        outputs = self.text_model(
             inputs_embeds=inputs_embeds,
             attention_mask=attention_mask,
             position_ids=position_ids,
             **kwargs,
         )
-
-        return BaseModelOutput(
-            last_hidden_state=hidden_states,
-        )
+        return outputs
 
 
 class T5Gemma2Decoder(T5Gemma2PreTrainedModel):
@@ -1031,7 +1029,8 @@ class T5Gemma2Decoder(T5Gemma2PreTrainedModel):
         self.rotary_emb = T5Gemma2RotaryEmbedding(config)
         self.post_init()
 
-    @check_model_inputs
+    @merge_with_config_defaults
+    @capture_outputs
     @auto_docstring
     def forward(
         self,
@@ -1069,7 +1068,7 @@ class T5Gemma2Decoder(T5Gemma2PreTrainedModel):
         if not isinstance(self_attn_mask_mapping := attention_mask, dict):
             mask_kwargs = {
                 "config": self.config,
-                "input_embeds": inputs_embeds,
+                "inputs_embeds": inputs_embeds,
                 "attention_mask": attention_mask,
                 "cache_position": cache_position,
                 "past_key_values": past_key_values.self_attention_cache if past_key_values is not None else None,
@@ -1086,7 +1085,7 @@ class T5Gemma2Decoder(T5Gemma2PreTrainedModel):
         if not isinstance(cross_attn_mask_mapping := encoder_attention_mask, dict):
             mask_kwargs = {
                 "config": self.config,
-                "input_embeds": encoder_hidden_states,
+                "inputs_embeds": encoder_hidden_states,
                 "attention_mask": encoder_attention_mask,
                 "cache_position": cache_position,
                 "past_key_values": None,
