@@ -176,12 +176,13 @@ else:
         _device = matQ.device
         nc = num_chunks
         batch_size, nh, dqk, dhv = matC_states.shape
-        matC_k_states = matC_states.view(batch_size, nh, nc, dqk // nc, dhv)
-        vecN_k_states = vecN_states.view(batch_size, nh, nc, dqk // nc)
+        dhqk = dqk // nc
+        matC_k_states = matC_states.view(batch_size, nh, nc, dhqk, dhv)
+        vecN_k_states = vecN_states.view(batch_size, nh, nc, dhqk)
         scaMinter_k_states = scaMinter_states
 
-        matQ = matQ.view(batch_size, nh, nc, chunk_size, dqk)
-        matK = matK.view(batch_size, nh, nc, chunk_size, dqk)
+        matQ = matQ.view(batch_size, nh, nc, chunk_size, dhqk)
+        matK = matK.view(batch_size, nh, nc, chunk_size, dhqk)
         matV = matV.view(batch_size, nh, nc, chunk_size, dhv)
 
         ltr = torch.tril(
@@ -232,7 +233,7 @@ else:
 
         # we need the denominator and the overall max state for the backward pass
         vecN_out = vecDenom_max_common.reshape(batch_size, nh, nc * chunk_size)
-        vecM_out = vecM_k_combine(batch_size, nh, nc * chunk_size)
+        vecM_out = vecM_k_combine.reshape(batch_size, nh, nc * chunk_size)
         return matH_out, vecN_out, vecM_out
 
     def mlstm_chunkwise_fw(
@@ -780,7 +781,7 @@ else:
             c_initial: torch.Tensor | None = None,
             n_initial: torch.Tensor | None = None,
             m_initial: torch.Tensor | None = None,
-            return_last_states: bool = False,
+            return_last_states: bool | None = None,
             mode: Literal["train", "inference"] | None = None,
         ) -> torch.Tensor | tuple[torch.Tensor, tuple[torch.Tensor, torch.Tensor, torch.Tensor]]:
             """Forward pass of the mLSTM backend.
